@@ -1,27 +1,23 @@
 # -*- coding: utf-8 -*-
 
-from os import path
-
 from django.core.files import File
 from storage import S3Storage
 
 
-def upload(filename, prefix=False, bucket_name=False, key=None, secret=None,
-    host=None):
+def upload(filename, name=None, prefix=False, bucket_name=False, key=None,
+           secret=None, host=None):
     """
     Uploading files to Amamzon S3.
     """
-    s3 = S3Storage(bucket_name=bucket_name, key=key, secret=secret, host=host)
-    name = None
-
     if isinstance(filename, basestring):
-        filename = open(filename, 'rb')
-        name = path.basename(filename)
+        fl = open(filename, 'rb')
     elif isinstance(filename, (file, File)):
-        name = filename.name
+        fl = filename
+    else:
+        raise TypeError('File must be file or string instance.')
 
     if not name:
-        raise TypeError('Filename must be file or string instance.')
+        name = fl.name
 
     if prefix:
         if prefix.endswith('/'):
@@ -31,6 +27,7 @@ def upload(filename, prefix=False, bucket_name=False, key=None, secret=None,
     else:
         full_path = name
 
-    key = s3.save(full_path, filename)
+    s3 = S3Storage(bucket_name=bucket_name, key=key, secret=secret, host=host)
+    s3.save(full_path, file)
 
     return s3.url(full_path)
